@@ -1,6 +1,6 @@
 #!/usr/bin/env python3
 # --------------------( LICENSE                            )--------------------
-# Copyright (c) 2014-2023 Beartype authors.
+# Copyright (c) 2014-2025 Beartype authors.
 # See "LICENSE" for further details.
 
 '''
@@ -12,7 +12,7 @@ This private submodule is *not* intended for importation by downstream callers.
 
 # ....................{ TODO                               }....................
 #FIXME: Generalize @callable_cached to revert to the body of the
-#@beartype._util.type.decorator.decmemo.func_cached decorator when the passed
+#betse._util.type.decorator.decmemo.func_cached decorator when the passed
 #callable accepts *NO* parameters, which can be trivially decided by inspecting
 #the code object of this callable. Why do this? Because the @func_cached
 #decorator is *INSANELY* fast for this edge case -- substantially faster than
@@ -20,27 +20,18 @@ This private submodule is *not* intended for importation by downstream callers.
 
 # ....................{ IMPORTS                            }....................
 from beartype.roar._roarexc import _BeartypeUtilCallableCachedException
-from beartype.typing import (
-    Dict,
-    TypeVar,
-)
+from beartype.typing import Dict
+from beartype._data.typing.datatyping import CallableT
 from beartype._util.func.arg.utilfuncargtest import (
     die_unless_func_args_len_flexible_equal,
     is_func_arg_variadic,
 )
 from beartype._util.text.utiltextlabel import label_callable
-from beartype._util.utilobject import SENTINEL
-from collections.abc import Callable
+from beartype._data.kind.datakindiota import SENTINEL
 from functools import wraps
 
-# ....................{ PRIVATE ~ hints                    }....................
-_CallableT = TypeVar('_CallableT', bound=Callable)
-'''
-Type variable bound to match *only* callables.
-'''
-
 # ....................{ DECORATORS ~ callable              }....................
-def callable_cached(func: _CallableT) -> _CallableT:
+def callable_cached(func: CallableT) -> CallableT:
     '''
     **Memoize** (i.e., efficiently re-raise all exceptions previously raised by
     the decorated callable when passed the same parameters (i.e., parameters
@@ -82,7 +73,7 @@ def callable_cached(func: _CallableT) -> _CallableT:
              #. Returns that value.
 
     Caveats
-    ----------
+    -------
     **The decorated callable must accept no keyword parameters.** While this
     decorator previously memoized keyword parameters, doing so incurred
     significant performance penalties defeating the purpose of caching. This
@@ -144,16 +135,16 @@ def callable_cached(func: _CallableT) -> _CallableT:
 
     Parameters
     ----------
-    func : _CallableT
+    func : CallableT
         Callable to be memoized.
 
     Returns
-    ----------
-    _CallableT
+    -------
+    CallableT
         Closure wrapping this callable with memoization.
 
     Raises
-    ----------
+    ------
     _BeartypeUtilCallableCachedException
         If this callable accepts a variadic positional parameter (e.g.,
         ``*args``).
@@ -165,6 +156,7 @@ def callable_cached(func: _CallableT) -> _CallableT:
     # below. For speed, this decorator violates DRY by duplicating logic.
     #!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
 
+    # ....................{ LOCALS                         }....................
     # Dictionary mapping a tuple of all flattened parameters passed to each
     # prior call of the decorated callable with the value returned by that call
     # if any (i.e., if that call did *NOT* raise an exception).
@@ -181,14 +173,15 @@ def callable_cached(func: _CallableT) -> _CallableT:
     # get() method of this dictionary, localized for efficiency.
     args_flat_to_exception_get = args_flat_to_exception.get
 
+    # ....................{ CLOSURE                        }....................
     @wraps(func)
     def _callable_cached(*args):
         f'''
         Memoized variant of the {func.__name__}() callable.
 
         See Also
-        ----------
-        :func:`callable_cached`
+        --------
+        :func:`.callable_cached`
             Further details.
         '''
 
@@ -228,15 +221,14 @@ def callable_cached(func: _CallableT) -> _CallableT:
             # If this callable previously raised an exception when called with
             # these parameters, re-raise the same exception.
             if exception:
-                raise exception  # pyright: ignore[reportGeneralTypeIssues]
+                raise exception  # pyright: ignore
             # Else, this callable either has yet to be called with these
             # parameters *OR* has but failed to raise an exception.
 
             # Value returned by a prior call to the decorated callable when
             # passed these parameters *OR* a sentinel placeholder otherwise
             # (i.e., if this callable has yet to be passed these parameters).
-            return_value = args_flat_to_return_value_get(
-                args_flat, SENTINEL)
+            return_value = args_flat_to_return_value_get(args_flat, SENTINEL)
 
             # If this callable has already been called with these parameters,
             # return the value returned by that prior call.
@@ -269,11 +261,21 @@ def callable_cached(func: _CallableT) -> _CallableT:
         # Return this value.
         return return_value
 
+    # ....................{ RETURN                         }....................
     # Return this wrapper.
     return _callable_cached  # type: ignore[return-value]
 
 # ....................{ DECORATORS ~ method                }....................
-def method_cached_arg_by_id(func: _CallableT) -> _CallableT:
+#FIXME: *BIG MISTAKE.* Woops. Under CPython, object IDs are (mostly) simply the
+#underlying C-based memory addresses of those objects. Ergo, object IDs are
+#unique *ONLY* for the duration of those objects.
+#
+#This makes this decorator fundamentally unsound. Refactor as follows, please:
+#* Replace the two usages of this decorator in the
+#  "beartype.door._cls.doorsuper" submodule with a manual caching scheme
+#  leveraging a global dictionary mapping from the name 
+#* Permanently excise this decorator.
+def method_cached_arg_by_id(func: CallableT) -> CallableT:
     '''
     **Memoize** (i.e., efficiently re-raise all exceptions previously raised by
     the decorated method when passed the same *exact* parameters (i.e.,
@@ -282,19 +284,19 @@ def method_cached_arg_by_id(func: _CallableT) -> _CallableT:
     rather than inefficiently recalling that method) the passed method.
 
     Caveats
-    ----------
+    -------
     **This decorator is only intended to decorate bound methods** (i.e., either
     class or instance methods bound to a class or instance). This decorator is
     *not* intended to decorate functions or static methods.
 
     **This decorator is only intended to decorate a method whose sole argument
     is guaranteed to be a memoized singleton** (e.g.,
-    :class:`beartype.door.TypeHint` singleton). In this case, the object
-    identifier of that argument uniquely identifies that argument across *all*
-    calls to that method -- enabling this decorator to memoize that method.
-    Conversely, if that argument is *not* guaranteed to be a memoized singleton,
-    this decorator will fail to memoize that method while wasting considerable
-    space and time attempting to do so. In short, care is warranted.
+    :class:`beartype.door.TypeHint` singleton). In this case, the object ID of
+    that argument uniquely identifies that argument across *all* calls to that
+    method -- enabling this decorator to memoize that method. Conversely, if
+    that argument is *not* guaranteed to be a memoized singleton, this decorator
+    will fail to memoize that method while wasting considerable space and time
+    attempting to do so. In short, care is warranted.
 
     This decorator is a micro-optimized variant of the more general-purpose
     :func:`callable_cached` decorator, which should be preferred in most cases.
@@ -333,16 +335,16 @@ def method_cached_arg_by_id(func: _CallableT) -> _CallableT:
 
     Parameters
     ----------
-    func : _CallableT
+    func : CallableT
         Callable to be memoized.
 
     Returns
-    ----------
-    _CallableT
+    -------
+    CallableT
         Closure wrapping this callable with memoization.
 
     Raises
-    ----------
+    ------
     _BeartypeUtilCallableCachedException
         If this callable accepts either:
 
@@ -351,15 +353,17 @@ def method_cached_arg_by_id(func: _CallableT) -> _CallableT:
         * A variadic positional parameter (e.g., ``*args``).
 
     See Also
-    ----------
+    --------
     :func:`callable_cached`
         Further details.
     '''
     assert callable(func), f'{repr(func)} not callable.'
 
+    # ....................{ IMPORTS                        }....................
     # Avoid circular import dependencies.
     from beartype._util.func.utilfuncwrap import unwrap_func_all
 
+    # ....................{ PREAMBLE                       }....................
     # Lowest-level wrappee callable wrapped by this wrapper callable.
     func_wrappee = unwrap_func_all(func)
 
@@ -375,7 +379,8 @@ def method_cached_arg_by_id(func: _CallableT) -> _CallableT:
     )
     # Else, this wrappee accepts exactly one flexible parameter.
 
-    # If this wrappee accepts variadic arguments, raise an exception.
+    # If this wrappee accepts variadic arguments (either positional or keyword),
+    # raise an exception.
     if is_func_arg_variadic(func_wrappee):
         raise _BeartypeUtilCallableCachedException(
             f'@method_cached_arg_by_id {label_callable(func)} '
@@ -388,6 +393,7 @@ def method_cached_arg_by_id(func: _CallableT) -> _CallableT:
     # speed, this decorator violates DRY by duplicating logic.
     #!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
 
+    # ....................{ LOCALS                         }....................
     # Dictionary mapping a tuple of all flattened parameters passed to each
     # prior call of the decorated callable with the value returned by that call
     # if any (i.e., if that call did *NOT* raise an exception).
@@ -404,13 +410,14 @@ def method_cached_arg_by_id(func: _CallableT) -> _CallableT:
     # get() method of this dictionary, localized for efficiency.
     args_flat_to_exception_get = args_flat_to_exception.get
 
+    # ....................{ CLOSURE                        }....................
     @wraps(func)
     def _method_cached(self_or_cls, arg):
         f'''
         Memoized variant of the {func.__name__}() callable.
 
         See Also
-        ----------
+        --------
         :func:`callable_cached`
             Further details.
         '''
@@ -441,15 +448,14 @@ def method_cached_arg_by_id(func: _CallableT) -> _CallableT:
             # If this callable previously raised an exception when called with
             # these parameters, re-raise the same exception.
             if exception:
-                raise exception  # pyright: ignore[reportGeneralTypeIssues]
+                raise exception  # pyright: ignore
             # Else, this callable either has yet to be called with these
             # parameters *OR* has but failed to raise an exception.
 
             # Value returned by a prior call to the decorated callable when
             # passed these parameters *OR* a sentinel placeholder otherwise
             # (i.e., if this callable has yet to be passed these parameters).
-            return_value = args_flat_to_return_value_get(
-                args_flat, SENTINEL)
+            return_value = args_flat_to_return_value_get(args_flat, SENTINEL)
 
             # If this callable has already been called with these parameters,
             # return the value returned by that prior call.
@@ -482,11 +488,12 @@ def method_cached_arg_by_id(func: _CallableT) -> _CallableT:
         # Return this value.
         return return_value
 
+    # ....................{ RETURN                         }....................
     # Return this wrapper.
     return _method_cached  # type: ignore[return-value]
 
 # ....................{ DECORATORS ~ property              }....................
-def property_cached(func: _CallableT) -> _CallableT:
+def property_cached(func: CallableT) -> CallableT:
     '''
     **Memoize** (i.e., efficiently cache and return all previously returned
     values of the passed property method as well as all previously raised
@@ -507,7 +514,7 @@ def property_cached(func: _CallableT) -> _CallableT:
     called at most once for each object exposing this property.
 
     Caveats
-    ----------
+    -------
     **This decorator must be preceded by an explicit usage of the standard**
     :class:`property` **decorator.** Although this decorator could be trivially
     refactored to automatically decorate the returned property method by the
@@ -545,8 +552,13 @@ def property_cached(func: _CallableT) -> _CallableT:
 
     Parameters
     ----------
-    func : _CallableT
+    func : CallableT
         Property method to be memoized.
+
+    Returns
+    -------
+    CallableT
+        Dynamically generated function wrapping this property with memoization.
     '''
     assert callable(func), f'{repr(func)} not callable.'
 
@@ -594,9 +606,9 @@ This prefix:
   filtering) to uniquely match and act upon these variables.
 * Is intentionally prefixed by double rather than single underscores (i.e.,
   ``"__"`` rather than ``"_"``), ensuring that our
-  :meth:`beartype._check.forward._fwdref._BeartypeForwardRefMeta.__getattr__`
+  :meth:`beartype._check.forward.reference.fwdrefmeta.BeartypeForwardRefMeta.__getattr__`
   dunder method ignores the private instance variables cached by our cached
-  :meth:`beartype._check.forward._fwdref._BeartypeForwardRefMeta.__beartype_type__`
+  :meth:`beartype._check.forward.reference.fwdrefmeta.BeartypeForwardRefMeta.__type_beartype__`
   property.
 '''
 
@@ -644,7 +656,7 @@ These statements include (in order):
   function definition time as the default value of an arbitrary parameter.
 
 Design
-----------
+------
 While there exist numerous alternative implementations for caching properties,
 the approach implemented below has been profiled to be the most efficient.
 Alternatives include (in order of decreasing efficiency):
