@@ -1,6 +1,6 @@
 #!/usr/bin/env python3
 # --------------------( LICENSE                            )--------------------
-# Copyright (c) 2014-2024 Beartype authors.
+# Copyright (c) 2014-2025 Beartype authors.
 # See "LICENSE" for further details.
 
 '''
@@ -12,14 +12,26 @@ This private submodule is *not* intended for importation by downstream callers.
 
 # ....................{ IMPORTS                            }....................
 from beartype.roar._roarexc import _BeartypeUtilCallableException
-from beartype.typing import Any
-from beartype._data.hint.datahintfactory import TypeGuard
-from beartype._data.hint.datahinttyping import (
+from beartype.typing import (
+    Any,
+    Optional,
+)
+from beartype._cave._cavefast import (
+    MethodBoundInstanceOrClassType,
+    # MethodDecoratorBuiltinTypes,
+    MethodDecoratorClassOrStaticTypes,
+    MethodDecoratorClassType,
+    MethodDecoratorPropertyType,
+    MethodDecoratorStaticType,
+)
+from beartype._data.typing.datatypingport import TypeIs
+from beartype._data.typing.datatyping import (
     Codeobjable,
+    # MethodDescriptorBuiltin,
     TypeException,
 )
-from beartype._util.cache.utilcachecall import callable_cached
-from beartype._util.func.arg.utilfuncargget import (
+# from beartype._util.cache.utilcachecall import callable_cached
+from beartype._util.func.arg.utilfuncarglen import (
     get_func_args_nonvariadic_len)
 from beartype._util.func.arg.utilfuncargtest import (
     is_func_arg_variadic_positional,
@@ -34,6 +46,7 @@ from inspect import (
 )
 
 # ....................{ CONSTANTS                          }....................
+#FIXME: Shift into the "beartype._data.func.datafunc" submodule, please.
 FUNC_NAME_LAMBDA = '<lambda>'
 '''
 Default name of all **pure-Python lambda functions** (i.e., function declared
@@ -55,7 +68,7 @@ AST-based parsing, which comes with its own substantial caveats, concerns,
 edge cases, and false positives. If you must pick your poison, pick this one.
 '''
 
-# ....................{ VALIDATORS                         }....................
+# ....................{ RAISERS                            }....................
 def die_unless_func_python(
     # Mandatory parameters.
     func: Codeobjable,
@@ -91,12 +104,12 @@ def die_unless_func_python(
 
     See Also
     --------
-    :func:`.is_func_python`
+    :func:`.is_func_codeobjable`
         Further details.
     '''
 
     # If that callable is *NOT* pure-Python, raise an exception.
-    if not is_func_python(func):
+    if not is_func_codeobjable(func):
         assert isinstance(exception_cls, type), (
             f'{repr(exception_cls)} not class.')
         assert issubclass(exception_cls, Exception), (
@@ -114,7 +127,121 @@ def die_unless_func_python(
             f'{exception_prefix}{repr(func)} not pure-Python function.')
     # Else, that callable is pure-Python.
 
-# ....................{ VALIDATORS ~ descriptors           }....................
+# ....................{ RAISERS ~ descriptors              }....................
+#FIXME: Unit test us up, please.
+def die_unless_func_boundmethod(
+    # Mandatory parameters.
+    func: Any,
+
+    # Optional parameters.
+    exception_cls: TypeException = _BeartypeUtilCallableException,
+    exception_prefix: str = '',
+) -> None:
+    '''
+    Raise an exception unless the passed object is a **C-based bound instance
+    method descriptor** callable implicitly instantiated and assigned on the
+    instantiation of an object whose class declares an instance function (whose
+    first parameter is typically named ``self``) as an instance variable of that
+    object such that that callable unconditionally passes that object as the
+    value of that first parameter on all calls to that callable).
+
+    Parameters
+    ----------
+    func : Any
+        Object to be inspected.
+    exception_cls : TypeException, optional
+        Type of exception to be raised. Defaults to
+        :class:`._BeartypeUtilCallableException`.
+    exception_prefix : str, optional
+        Human-readable label prefixing the representation of this object in the
+        exception message. Defaults to the empty string.
+
+    Raises
+    ------
+    exception_cls
+         If the passed object is *not* a bound method descriptor.
+
+    See Also
+    --------
+    :func:`.is_func_boundmethod`
+        Further details.
+    '''
+
+    # If this object is *NOT* a bound method descriptor, raise an exception.
+    if not is_func_boundmethod(func):
+        assert isinstance(exception_cls, type), (
+            f'{repr(exception_cls)} not class.')
+        assert issubclass(exception_cls, Exception), (
+            f'{repr(exception_cls)} not exception subclass.')
+        assert isinstance(exception_prefix, str), (
+            f'{repr(exception_prefix)} not string.')
+
+        # Raise a human-readable exception.
+        raise exception_cls(
+            f'{exception_prefix}{repr(func)} not '
+            f'C-based bound instance method descriptor.'
+        )
+    # Else, this object is a bound method descriptor.
+
+
+#FIXME: Unit test us up, please.
+def die_unless_func_class_or_static_method(
+    # Mandatory parameters.
+    func: Any,
+
+    # Optional parameters.
+    exception_cls: TypeException = _BeartypeUtilCallableException,
+    exception_prefix: str = '',
+) -> None:
+    '''
+    Raise an exception unless the passed object is a **C-based unbound class or
+    static method descriptor** (i.e., method decorated by either the builtin
+    :class:`classmethod` or :class:`staticmethod` decorators, yielding a
+    non-callable instance of that decorator class implemented in low-level C and
+    accessible via the low-level :attr:`object.__dict__` dictionary rather than
+    as class or instance attributes).
+
+    Parameters
+    ----------
+    func : Any
+        Object to be inspected.
+    exception_cls : TypeException, optional
+        Type of exception to be raised. Defaults to
+        :exc:`._BeartypeUtilCallableException`.
+    exception_prefix : str, optional
+        Human-readable substring prefixing raised exception messages. Defaults
+        to the empty string.
+
+    Raises
+    ------
+    exception_cls
+         If the passed object is *not* a class or static method descriptor.
+
+    See Also
+    --------
+    :func:`.is_func_classmethod`
+        Further details.
+    '''
+
+    # If this object is neither a class *NOR* static method descriptor, raise an
+    # exception.
+    if not isinstance(func, MethodDecoratorClassOrStaticTypes):
+        assert isinstance(exception_cls, type), (
+            f'{repr(exception_cls)} not class.')
+        assert issubclass(exception_cls, Exception), (
+            f'{repr(exception_cls)} not exception subclass.')
+        assert isinstance(exception_prefix, str), (
+            f'{repr(exception_prefix)} not string.')
+
+        # Raise a human-readable exception.
+        raise exception_cls(
+            f'{exception_prefix}{repr(func)} not '
+            f'C-based unbound @classmethod or @staticmethod descriptor.'
+        )
+    # Else, this object is a class or static method descriptor.
+
+
+#FIXME: Currently unused, but extensively tested. *shrug*
 def die_unless_func_classmethod(
     # Mandatory parameters.
     func: Any,
@@ -224,6 +351,7 @@ def die_unless_func_property(
     # Else, this object is a property method descriptor.
 
 
+#FIXME: Currently unused, but extensively tested. *shrug*
 def die_unless_func_staticmethod(
     # Mandatory parameters.
     func: Any,
@@ -279,7 +407,34 @@ def die_unless_func_staticmethod(
     # Else, this object is a static method descriptor.
 
 # ....................{ TESTERS                            }....................
-def is_func_lambda(func: Any) -> TypeGuard[Callable]:
+def is_func_codeobjable(func: object) -> TypeIs[Callable]:
+    '''
+    :data:`True` only if the passed object is **code-objectable** (i.e., either
+    a pure-Python callable, low-level code object underlying a pure-Python
+    callable, or related object encapsulating such a low-level code object).
+
+    This tester effectively tests whether this object is a **pure-Python
+    callable** (i.e., implemented in Python as either a function or method
+    rather than in C as either a builtin bundled with the active Python
+    interpreter *or* third-party C extension function).
+
+    Parameters
+    ----------
+    func : object
+        Object to be inspected.
+
+    Returns
+    -------
+    bool
+        :data:`True` only if this object is code-objectable.
+    '''
+
+    # Return true only if a pure-Python code object underlies this object.
+    # C-based callables are associated with *NO* code objects.
+    return get_func_codeobj_or_none(func) is not None
+
+
+def is_func_lambda(func: Any) -> TypeIs[Callable]:
     '''
     :data:`True` only if the passed object is a **pure-Python lambda function**
     (i.e., function declared as a ``lambda`` expression embedded in a larger
@@ -299,7 +454,7 @@ def is_func_lambda(func: Any) -> TypeGuard[Callable]:
     # Return true only if this both...
     return (
         # This callable is pure-Python *AND*...
-        is_func_python(func) and
+        is_func_codeobjable(func) and
         # This callable's name is the lambda-specific placeholder name
         # initially given by Python to *ALL* lambda functions. Technically,
         # this name may be externally changed by malicious third parties after
@@ -313,13 +468,26 @@ def is_func_lambda(func: Any) -> TypeGuard[Callable]:
         func.__name__ == FUNC_NAME_LAMBDA
     )
 
-
-def is_func_python(func: object) -> TypeGuard[Callable]:
+# ....................{ TESTERS ~ descriptor               }....................
+#FIXME: Unit test us up, please.
+def is_func_boundmethod(func: Any) -> TypeIs[MethodBoundInstanceOrClassType]:
     '''
-    :data:`True` only if the passed object is a **pure-Python callable** (i.e.,
-    implemented in Python as either a function or method rather than in C as
-    either a builtin bundled with the active Python interpreter *or*
-    third-party C extension function).
+    :data:`True` only if the passed object is a **C-based bound instance method
+    descriptor** (i.e., callable implicitly instantiated and assigned on the
+    instantiation of an object whose class declares an instance function (whose
+    first parameter is typically named ``self``) as an instance variable of that
+    object such that that callable unconditionally passes that object as the
+    value of that first parameter on all calls to that callable).
+
+    Caveats
+    -------
+    Instance method objects are *only* directly accessible as instance
+    attributes. When accessed as either class attributes *or* via the low-level
+    :attr:`object.__dict__` dictionary, instance methods are only functions
+    (i.e., instances of the standard :class:`beartype.cave.FunctionType` type).
+
+    Instance method objects are callable. Indeed, the callability of instance
+    method objects is the entire point of instance method objects.
 
     Parameters
     ----------
@@ -329,32 +497,62 @@ def is_func_python(func: object) -> TypeGuard[Callable]:
     Returns
     -------
     bool
-        :data:`True` only if this object is a pure-Python callable.
+        :data:`True` only if this object is a C-based bound instance method
+        descriptor.
     '''
 
-    # Return true only if a pure-Python code object underlies this object.
-    # C-based callables are associated with *NO* code objects.
-    return get_func_codeobj_or_none(func) is not None
+    # Only the penitent one-liner shall pass.
+    return isinstance(func, MethodBoundInstanceOrClassType)
 
-# ....................{ TESTERS ~ descriptor               }....................
-def is_func_classmethod(func: Any) -> TypeGuard[classmethod]:
+# ....................{ TESTERS ~ descriptor : uncallable  }....................
+#FIXME: Unit test us up, please.
+#FIXME: Currently unused but preserved for posterity. We'll want this someday!
+# def is_func_class_property_or_static_method(func: Any)  -> TypeIs[
+#     MethodDescriptorBuiltin]:
+#     '''
+#     :data:`True` only if the passed object is an **unbound class, property, or
+#     static method descriptor** (i.e., C-based decorator type builtin to Python
+#     whose instance is typically uncallable but encapsulates a callable
+#     pure-Python method).
+#
+#     These method objects are *not* callable, as their implementations fail to
+#     define the ``__call__()`` dunder method.
+#
+#     Parameters
+#     ----------
+#     func : object
+#         Object to be inspected.
+#
+#     Returns
+#     -------
+#     bool
+#         :data:`True` only if this object is an unbound class, property, or
+#         static method descriptor.
+#     '''
+#
+#     # Line up for the one-liner you never knew you needed in your life.
+#     return isinstance(func, MethodDecoratorBuiltinTypes)
+
+
+def is_func_classmethod(func: Any) -> TypeIs[MethodDecoratorClassType]:
     '''
-    :data:`True` only if the passed object is a **C-based unbound class method
+    :data:`True` only if the passed object is an **unbound class method
     descriptor** (i.e., method decorated by the builtin :class:`classmethod`
     decorator, yielding a non-callable instance of that :class:`classmethod`
-    decorator class implemented in low-level C and accessible via the
-    low-level :attr:`object.__dict__` dictionary rather than as class or
-    instance attributes).
+    decorator class implemented in low-level C and accessible via the low-level
+    :attr:`object.__dict__` dictionary rather than as class or instance
+    attributes).
 
     Caveats
     -------
     Class method objects are *only* directly accessible via the low-level
     :attr:`object.__dict__` dictionary. When accessed as class or instance
-    attributes, class methods reduce to instances of the standard
-    :class:`MethodBoundInstanceOrClassType` type.
+    attributes, class methods are indistinguishable from **bound method
+    descriptors** (i.e., :class:`MethodBoundInstanceOrClassType` instances)
+    bound to that class.
 
     Class method objects are *not* callable, as their implementations fail to
-    define the ``__call__`` dunder method.
+    define the ``__call__()`` dunder method.
 
     Parameters
     ----------
@@ -364,15 +562,14 @@ def is_func_classmethod(func: Any) -> TypeGuard[classmethod]:
     Returns
     -------
     bool
-        :data:`True` only if this object is a C-based unbound class method
-        descriptor.
+        :data:`True` only if this object is an unbound class method descriptor.
     '''
 
     # Now you too have seen the pure light of the one-liner.
-    return isinstance(func, classmethod)
+    return isinstance(func, MethodDecoratorClassType)
 
 
-def is_func_property(func: Any) -> TypeGuard[property]:
+def is_func_property(func: Any) -> TypeIs[MethodDecoratorPropertyType]:
     '''
     :data:`True` only if the passed object is a **C-based unbound property
     method descriptor** (i.e., method decorated by the builtin :class:`property`
@@ -401,10 +598,10 @@ def is_func_property(func: Any) -> TypeGuard[property]:
     '''
 
     # We rejoice in the shared delight of one-liners.
-    return isinstance(func, property)
+    return isinstance(func, MethodDecoratorPropertyType)
 
 
-def is_func_staticmethod(func: Any) -> TypeGuard[staticmethod]:
+def is_func_staticmethod(func: Any) -> TypeIs[MethodDecoratorStaticType]:
     '''
     :data:`True` only if the passed object is a **C-based unbound static method
     descriptor** (i.e., method decorated by the builtin :class:`staticmethod`
@@ -435,10 +632,10 @@ def is_func_staticmethod(func: Any) -> TypeGuard[staticmethod]:
     '''
 
     # Does the one-liner have Buddhahood? Mu.
-    return isinstance(func, staticmethod)
+    return isinstance(func, MethodDecoratorStaticType)
 
 # ....................{ TESTERS ~ async                    }....................
-def is_func_async(func: object) -> TypeGuard[Callable]:
+def is_func_async(func: object) -> TypeIs[Callable]:
     '''
     :data:`True` only if the passed object is an **asynchronous callable
     factory** (i.e., awaitable factory callable implicitly creating and
@@ -500,7 +697,7 @@ def is_func_async(func: object) -> TypeGuard[Callable]:
     )
 
 
-def is_func_coro(func: object) -> TypeGuard[Callable]:
+def is_func_coro(func: object) -> TypeIs[Callable]:
     '''
     :data:`True` only if the passed object is an **asynchronous coroutine
     factory** (i.e., awaitable callable containing *no* ``yield`` expression
@@ -538,7 +735,7 @@ def is_func_coro(func: object) -> TypeGuard[Callable]:
     )
 
 
-def is_func_async_generator(func: object) -> TypeGuard[Callable]:
+def is_func_async_generator(func: object) -> TypeIs[Callable]:
     '''
     :data:`True` only if the passed object is an **asynchronous generator
     factory** (i.e., awaitable callable containing one or more ``yield``
@@ -576,7 +773,7 @@ def is_func_async_generator(func: object) -> TypeGuard[Callable]:
     )
 
 # ....................{ TESTERS ~ sync                     }....................
-def is_func_sync_generator(func: object) -> TypeGuard[Callable]:
+def is_func_sync_generator(func: object) -> TypeIs[Callable]:
     '''
     :data:`True` only if the passed object is an **synchronous generator
     factory** (i.e., awaitable callable containing one or more ``yield``
@@ -711,7 +908,7 @@ def is_func_nested(func: Callable) -> bool:
     )
 
 # ....................{ TESTERS ~ nested : closure         }....................
-def is_func_closure(func: Any) -> TypeGuard[Callable]:
+def is_func_closure(func: Any) -> TypeIs[Callable]:
     '''
     :data:`True` only if the passed callable is a **closure** (i.e., nested
     callable accessing one or more variables declared by the parent callable
@@ -741,7 +938,7 @@ def is_func_closure(func: Any) -> TypeGuard[Callable]:
     return getattr(func, '__closure__', None) is not None
 
 # ....................{ TESTERS ~ wrapper                  }....................
-def is_func_wrapper(func: Any) -> TypeGuard[Callable]:
+def is_func_wrapper(func: Any) -> TypeIs[Callable]:
     '''
     :data:`True` only if the passed object is a **callable wrapper** (i.e.,
     callable decorated by the standard :func:`functools.wraps` decorator for
@@ -776,8 +973,13 @@ def is_func_wrapper(func: Any) -> TypeGuard[Callable]:
     return hasattr(func, '__wrapped__')
 
 
-@callable_cached
-def is_func_wrapper_isomorphic(func: Any) -> TypeGuard[Callable]:
+def is_func_wrapper_isomorphic(
+    # Mandatory parameters.
+    func: Any,
+
+    # Optional parameters.
+    wrapper: Optional[Callable] = None,
+) -> TypeIs[Callable]:
     '''
     :data:`True` only if the passed object is an **isomorphic wrapper** (i.e.,
     callable decorated by the standard :func:`functools.wraps` decorator for
@@ -790,7 +992,14 @@ def is_func_wrapper_isomorphic(func: Any) -> TypeGuard[Callable]:
     decorated by an isomorphic decorator, which constitutes *most* real-world
     decorators of interest.
 
-    This tester is memoized for efficiency.
+    This tester is currently *not* memoized for efficiency, despite performing a
+    relatively non-trivial (albeit technically :math:`O(1)`) operation. Why?
+    Because this tester should typically be called at most once by the parent
+    :func:`beartype._util.func.utilfuncwrap.unwrap_func_all_isomorphic`
+    function, which is currently:
+
+    * The *only* other function calling this tester.
+    * Itself currently unmemoized.
 
     Caveats
     -------
@@ -802,6 +1011,21 @@ def is_func_wrapper_isomorphic(func: Any) -> TypeGuard[Callable]:
     ----------
     func : object
         Object to be inspected.
+    wrapper : Optional[Callable]
+        Wrapper callable to be unwrapped in the event that the callable to be
+        inspected for isomorphism differs from the callable to be unwrapped.
+        Typically, these two callables are the same. Edge cases in which these
+        two callables differ include:
+
+        * When ``wrapper`` is a **pseudo-callable** (i.e., otherwise uncallable
+          object whose type renders that object callable by defining the
+          ``__call__()`` dunder method) *and* ``func`` is that ``__call__()``
+          dunder method. If that pseudo-callable wraps a lower-level callable,
+          then that pseudo-callable (rather than ``__call__()`` dunder method)
+          defines the ``__wrapped__`` instance variable providing that callable.
+
+        Defaults to :data:`None`, in which case this parameter *actually*
+        defaults to ``func``.
 
     Returns
     -------
@@ -809,24 +1033,83 @@ def is_func_wrapper_isomorphic(func: Any) -> TypeGuard[Callable]:
         :data:`True` only if this object is an isomorphic decorator wrapper.
     '''
 
-    # If the passed callable is *NOT* a wrapper, immediately return false.
-    if not is_func_wrapper(func):
+    # If the caller failed to explicitly pass a callable to be unwrapped,
+    # default the callable to be unwrapped to the passed callable.
+    if wrapper is None:
+        wrapper = func
+    # Else, the caller explicitly passed a callable to be unwrapped. In this
+    # case, preserve that callable as is.
+
+    # If that callable is *NOT* a wrapper, immediately return false.
+    if not is_func_wrapper(wrapper):
         return False
     # Else, that callable is a wrapper.
 
+    # Number of non-variadic arguments permitted for this wrapper if isomorphic,
+    # defaulting to 0.
+    func_args_nonvariadic_len = 0
+
+    # If this object is a C-based bound method descriptor...
+    if is_func_boundmethod(func):
+        # Avoid circular import dependencies.
+        from beartype._util.func.utilfuncwrap import (
+            unwrap_func_boundmethod_once)
+        # print(f'Detecting bound method f{repr(func)} isomorphism...')
+
+        # Unwrap this descriptor to the pure-Python callable encapsulated by
+        # this descriptor.
+        func = unwrap_func_boundmethod_once(func)
+
+        # Permit this pure-Python callable to accept exactly one non-variadic
+        # argument, typically named "self" whose value is the object to which
+        # this bound method descriptor was bound at object instantiation time.
+        func_args_nonvariadic_len = 1
+    # Else, this object is *NOT* a C-based bound method descriptor.
+
     # Code object underlying that callable as is (rather than possibly unwrapped
     # to another code object entirely) if that callable is pure-Python *OR*
     # "None" otherwise (i.e., if that callable is C-based).
     func_codeobj = get_func_codeobj_or_none(func)
 
+    # If that callable is C-based...
+    if not func_codeobj:  # pragma: no cover
+        # print(f'Detecting C-based callable {repr(func)} isomorphism...')
+        # Return true only if that C-based callable is the __call__() dunder
+        # method of a pseudo-callable parent object. Although this tester
+        # *CANNOT* positively decide whether that object is isomorphic or not,
+        # *ALMOST* all __call__() dunder methods are C-based. Technically, this
+        # *COULD* constitute a false positive in various edge cases.
+        # Pragmatically, the alternatives are all worse. Blindly rejecting *ALL*
+        # C-based __call__() dunder methods as non-isomorphic would effectively
+        # prevent @beartype from decorating numerous pseudo-callable objects of
+        # interest, including:
+        # * Pseudo-callables dynamically generated by the third-party
+        #   "@jax.jit" decorator: e.g.,
+        #       from beartype import beartype
+        #       from jax import jit
+        #
+        #       # The @jax.jit decorator creates and returns a C-based
+        #       # pseudo-callable object defining an isomorphic __call__()
+        #       # dunder method. If this tester erroneously rejected that method
+        #       # as non-isomorphic, @beartype would be unable to decorate these
+        #       # pseudo-callable objects! Clearly, that would be bad.
+        #       @beartype
+        #       @jit
+        #       def muh_func(muh_arg: int) -> int:
+        #           return muh_arg
+        return func.__name__ == '__call__'
+    # Else, that callable is pure-Python.
+
     # Return true only if...
     return (
-        # That callable is pure-Python *AND*...
-        func_codeobj is not None and
-        # That callable accepts a variadic positional argument *AND*...
-        is_func_arg_variadic_positional(func_codeobj) and
-        # That callable accepts a variadic keyword argument *AND*...
-        is_func_arg_variadic_keyword(func_codeobj) and
-        # That callable accepts *NO* non-variadic arguments.
-        get_func_args_nonvariadic_len(func_codeobj) == 0
+        # That callable accepts no non-variadic arguments *AND*...
+        (
+            get_func_args_nonvariadic_len(func_codeobj) ==
+            func_args_nonvariadic_len
+        ) and
+        # That callable accepts variadic positional and/or keyword arguments.
+        (
+            is_func_arg_variadic_positional(func_codeobj) or
+            is_func_arg_variadic_keyword(func_codeobj)
+        )
     )
